@@ -9,41 +9,40 @@
 
 from __future__ import absolute_import, division, unicode_literals
 
+import re
 import sys
 
 from mo_future import (
     binary_type,
     generator_types,
     is_binary,
-    is_text,
     text,
     OrderedDict,
     none_type,
 )
 from mo_imports import export
 
-from mo_dots.datas import Data, SLOT, data_types, is_data
+from mo_dots.datas import Data, data_types, is_data
 from mo_dots.lists import (
     FlatList,
     is_list,
     is_sequence,
     is_container,
     is_many,
-    LIST,
     list_types,
     container_types,
     finite_types,
 )
 from mo_dots.nones import Null, NullType
 from mo_dots.objects import DataObject
-from mo_dots.utils import CLASS, OBJ, get_logger, get_module
+from mo_dots.utils import CLASS, SLOT, get_logger, get_module
 
 _module_type = type(sys.modules[__name__])
 _builtin_zip = zip
 _get = object.__getattribute__
 _set = object.__setattr__
 _new = object.__new__
-
+_dict_zip = zip
 
 ROOT_PATH = ["."]
 
@@ -71,22 +70,42 @@ def coalesce(*args):
 def zip(keys, values):
     """
     CONVERT LIST OF KEY/VALUE PAIRS TO Data
-    PLEASE `import dot`, AND CALL `dot.zip()`
+    PLEASE `import mo_dots`, AND CALL `mo_dots.zip()`
     """
     output = Data()
-    for i, k in enumerate(keys):
-        if i >= len(values):
-            break
-        output[k] = values[i]
+    for k, v in _dict_zip(keys, values):
+        output[k] = v
     return output
 
 
 def missing(value):
-    return value == None or value == ""
+    raise NotImplementedError("use is_missing")
+
+
+def is_missing(t):
+    # RETURN True IF EFFECTIVELY NOTHING
+    class_ = t.__class__
+    if class_ in null_types:
+        return True
+    elif class_ in data_types:
+        return False
+    elif class_ in finite_types and not t:
+        return True
+    elif class_ is text and not t:
+        return True
+    else:
+        return t == None
 
 
 def exists(value):
-    return value != None and value != ""
+    return not is_missing(value)
+
+
+ESCAPE_DOTS1 = re.compile(r"(^\.|\.$)")  # DOTS AT START/END
+ESCAPE_DOTS2 = re.compile(r"(?<!^)\.(?!$)")  # INTERNAL DOTS
+ILLEGAL_DOTS = re.compile(r"[^.]\.(?:\.\.)+")  # ODD DOTS ARE NOT ALLOWED
+SPLIT_DOTS = re.compile(r"(?<!\.)\.(?!\.)")  # SINGLE DOTS
+UNESCAPE_DOTS = re.compile(r"\x08|(?:\.\.)")  # ENCODED DOTS
 
 
 def literal_field(field):
@@ -94,7 +113,7 @@ def literal_field(field):
     RETURN SAME WITH DOTS (`.`) ESCAPED
     """
     try:
-        return field.replace(".", "\\.")
+        return ESCAPE_DOTS2.sub("..", ESCAPE_DOTS1.sub("\b", field))
     except Exception as e:
         get_logger().error("bad literal", e)
 
@@ -109,9 +128,7 @@ def unliteral_field(field):
     :param field: THE STRING TO DE-literal IZE
     :return: SIMPLER STRING
     """
-    if len(split_field(field)) > 1:
-        get_logger().error("Bad call! Dude!")
-    return field.replace("\\.", ".")
+    return UNESCAPE_DOTS.sub(".", field)
 
 
 def tail_field(field):
@@ -122,14 +139,8 @@ def tail_field(field):
     if field == "." or field == None:
         return ".", "."
     elif "." in field:
-        if "\\." in field:
-            path = field.replace("\\.", "\a").replace("\b", "\\.").split(".", 1)
-            if len(path) == 1:
-                return path[0].replace("\a", "."), "."
-            else:
-                return tuple(k.replace("\a", ".") for k in path)
-        else:
-            return field.split(".", 1)
+        path = split_field(field)
+        return path[0], join_field(path[1:])
     else:
         return field, "."
 
@@ -138,19 +149,17 @@ def split_field(field):
     """
     RETURN field AS ARRAY OF DOT-SEPARATED FIELDS
     """
-    if field == "." or field == None:
-        return []
-    elif is_text(field) and ("." in field or "\b" in field):
+    if ILLEGAL_DOTS.search(field):
+        get_logger().error("Odd number of dots is not allowed")
+    try:
         if field.startswith(".."):
             remainder = field.lstrip(".")
             back = len(field) - len(remainder) - 1
-            return [-1] * back + [
-                k.replace("\a", ".") for k in remainder.replace("\\.", "\a").replace("\b", "\\.").split(".")
-            ]
+            return [-1] * back + [UNESCAPE_DOTS.sub(".", k) for k in SPLIT_DOTS.split(remainder) if k]
         else:
-            return [k.replace("\a", ".") for k in field.replace("\\.", "\a").replace("\b", "\\.").split(".")]
-    else:
-        return [field]
+            return [UNESCAPE_DOTS.sub(".", k) for k in SPLIT_DOTS.split(field) if k]
+    except Exception as cause:
+        return []
 
 
 def join_field(path):
@@ -167,18 +176,11 @@ def join_field(path):
         for i, step in enumerate(path):
             if step != -1:
                 parents = "." + ("." * i)
-                return parents + ".".join([
-                    f.replace(".", "\a") for f in path[i:] if f != None
-                ]).replace("\\.", "\b").replace("\a", "\\.")
+                return parents + ".".join(literal_field(f) for f in path if f != None)
         return "." + ("." * len(path))
-    output = ".".join(f.replace(".", "\a") for f in path if f != None).replace("\\.", "\b").replace("\a", "\\.")
+    output = ".".join(literal_field(f) for f in path if f != None)
     return output if output else "."
 
-    # potent = [f for f in path if f != "."]
-    # if not potent:
-    #     return "."
-    # return ".".join([f.replace(".", "\\.") for f in potent])
-
 
 def concat_field(prefix, suffix):
     if suffix.startswith(".."):
@@ -247,7 +249,7 @@ def hash_value(v):
 
 def fromkeys(keys, value=None):
     if value == None:
-        return dict_to_data({})
+        return Data()
     return dict_to_data(dict.fromkeys(keys, value))
 
 
@@ -325,26 +327,6 @@ def _all_default(d, default, seen=None):
                 _all_default(existing_value, default_value, seen)
 
 
-def _get_dict_default(obj, key):
-    """
-    obj MUST BE A DICT
-    key IS EXPECTED TO BE LITERAL (NO ESCAPING)
-    TRY BOTH ATTRIBUTE AND ITEM ACCESS, OR RETURN Null
-    """
-    try:
-        return obj[key]
-    except Exception as f:
-        pass
-
-    try:
-        if float(key) == round(float(key), 0):
-            return obj[int(key)]
-    except Exception as f:
-        pass
-
-    return NullType(obj, key)
-
-
 def _getdefault(obj, key):
     """
     obj ANY OBJECT
@@ -534,7 +516,7 @@ def lower_match(value, candidates):
 
 def dict_to_data(d):
     """
-    DO NOT CHECK TYPE
+    FASTEST WAY TO MAKE Data, DO NOT CHECK TYPE
     :param d: dict
     :return: Data
     """
@@ -548,7 +530,7 @@ def list_to_data(v):
     to_data, BUT WITHOUT CHECKS
     """
     output = _new(FlatList)
-    _set(output, LIST, v)
+    _set(output, SLOT, v)
     return output
 
 
@@ -612,7 +594,7 @@ def _leaves_to_data(value):
                 key = key.decode("utf8")
 
             d = output
-            if key.find(".") == -1:
+            if "." not in key:
                 if value is None:
                     d.pop(key, None)
                 else:
@@ -649,13 +631,9 @@ def from_data(v):
         d = _get(v, SLOT)
         return d
     elif _type is FlatList:
-        return v.list
+        return _get(v, SLOT)
     elif _type is DataObject:
-        d = _get(v, OBJ)
-        if _get(d, CLASS) in data_types:
-            return d
-        else:
-            return v
+        return _get(v, SLOT)
     elif _type in generator_types:
         return (from_data(vv) for vv in v)
     else:
@@ -733,7 +711,7 @@ def tuplewrap(value):
 def is_null(t):
     # RETURN True IF EFFECTIVELY NOTHING
     class_ = t.__class__
-    if class_ in (none_type, NullType):
+    if class_ in null_types:
         return True
     else:
         try:
@@ -742,21 +720,24 @@ def is_null(t):
             return False
 
 
-def is_missing(t):
-    # RETURN True IF EFFECTIVELY NOTHING
+def is_not_null(t):
+    # RETURN True IF EFFECTIVELY SOMETHING
     class_ = t.__class__
-    if class_ in (none_type, NullType):
-        return True
-    elif class_ in data_types:
+    if class_ in null_types:
         return False
-    elif class_ in finite_types and not t:
+    elif class_ in data_types:
+        return True
+    elif class_ in finite_types and t:
         return True
     else:
-        return t == None
+        return t != None
 
 
+null_types = (none_type, NullType)
+
 # EXPORT
 export("mo_dots.nones", to_data)
+export("mo_dots.nones", null_types)
 
 export("mo_dots.datas", list_to_data)
 export("mo_dots.datas", dict_to_data)
@@ -767,6 +748,7 @@ export("mo_dots.datas", _getdefault)
 export("mo_dots.datas", hash_value)
 export("mo_dots.datas", listwrap)
 export("mo_dots.datas", literal_field)
+export("mo_dots.datas", null_types)
 
 export("mo_dots.lists", list_to_data)
 export("mo_dots.lists", to_data)
