@@ -10,11 +10,11 @@
 from __future__ import absolute_import, division, unicode_literals
 
 from mo_dots.lists import is_sequence
-from mo_dots.utils import CLASS, OBJ, KEY
-from mo_future import is_binary, text, none_type
+from mo_dots.utils import CLASS, KEY, SLOT
+from mo_future import is_binary, text
 from mo_imports import expect, export
 
-to_data = expect("to_data")
+to_data, null_types = expect("to_data", "null_types")
 
 _get = object.__getattribute__
 _set = object.__setattr__
@@ -31,14 +31,15 @@ class NullType(object):
     Null INSTANCES WILL TRACK THEIR OWN DEREFERENCE PATH SO
     ASSIGNMENT CAN BE DONE
     """
-    __slots__ = [OBJ, KEY]
+
+    __slots__ = [SLOT, KEY]
 
     def __init__(self, obj=None, key=None):
         """
         obj - VALUE BEING DEREFERENCED
         key - THE dict ITEM REFERENCE (DOT(.) IS NOT ESCAPED)
         """
-        _set(self, OBJ, obj)
+        _set(self, SLOT, obj)
         _set(self, KEY, key)
 
     def __bool__(self):
@@ -48,7 +49,7 @@ class NullType(object):
         return None
 
     def __float__(self):
-        return None
+        return Null
 
     def __nonzero__(self):
         return False
@@ -66,7 +67,7 @@ class NullType(object):
 
     def __iadd__(self, other):
         try:
-            o = _get(self, OBJ)
+            o = _get(self, SLOT)
             if o is None:
                 return self
             key = _get(self, KEY)
@@ -123,7 +124,7 @@ class NullType(object):
 
     def __eq__(self, other):
         class_ = _get(other, CLASS)
-        if class_ in (none_type, NullType):
+        if class_ in null_types:
             return True
         elif class_ is list and not other:
             return True
@@ -189,7 +190,7 @@ class NullType(object):
     def __getattr__(self, key):
         key = text(key)
 
-        o = to_data(_get(self, OBJ))
+        o = to_data(_get(self, SLOT))
         k = _get(self, KEY)
         if o == None:
             return NullType(self, key)
@@ -205,13 +206,13 @@ class NullType(object):
 
     def __setattr__(self, key, value):
         key = text(key)
-        o = _get(self, OBJ)
+        o = _get(self, SLOT)
         k = _get(self, KEY)
         seq = [k] + [key]
         _assign_to_null(o, seq, value)
 
     def __setitem__(self, key, value):
-        o = _get(self, OBJ)
+        o = _get(self, SLOT)
         if o is None:
             return
         k = _get(self, KEY)
@@ -257,7 +258,7 @@ def _assign_to_null(obj, path, value, force=True):
         if obj is Null:
             return
         if _get(obj, CLASS) is NullType:
-            o = _get(obj, OBJ)
+            o = _get(obj, SLOT)
             p = _get(obj, KEY)
             s = [p] + path
             return _assign_to_null(o, s, value)
@@ -290,7 +291,7 @@ def _split_field(field):
     if field == ".":
         return []
     else:
-        return [k.replace("\a", ".") for k in field.replace("\\.", "\a".replace("\b", "\\.")).split(".")]
+        return [k.replace("\b", ".") for k in field.replace("..", "\b").split(".")]
 
 
 def _setdefault(obj, key, value):
