@@ -12,16 +12,15 @@ from __future__ import absolute_import, division, unicode_literals
 from datetime import date, datetime
 from decimal import Decimal
 
-from mo_dots.datas import register_data, Data, SLOT
+from mo_dots.datas import register_data, Data
 from mo_dots.lists import FlatList
 from mo_dots.nones import NullType, Null
-from mo_dots.utils import CLASS, OBJ
+from mo_dots.utils import CLASS, SLOT
 from mo_future import (
     binary_type,
     generator_types,
     get_function_arguments,
     get_function_defaults,
-    none_type,
     text,
     Mapping,
 )
@@ -42,32 +41,39 @@ class DataObject(Mapping):
     TREAT AN OBJECT LIKE DATA
     """
 
+    __slots__ = [SLOT]
+
     def __init__(self, obj):
-        _set(self, OBJ, obj)
+        _set(self, SLOT, obj)
 
     def __getattr__(self, item):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         output = get_attr(obj, item)
         return datawrap(output)
 
     def __setattr__(self, key, value):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         set_attr(obj, key, value)
 
     def __getitem__(self, item):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
+        output = get_attr(obj, item)
+        return datawrap(output)
+
+    def get(self, item):
+        obj = _get(self, SLOT)
         output = get_attr(obj, item)
         return datawrap(output)
 
     def keys(self):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         try:
             return obj.__dict__.keys()
         except Exception as e:
             raise e
 
     def items(self):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         try:
             return obj.__dict__.items()
         except Exception as e:
@@ -76,7 +82,7 @@ class DataObject(Mapping):
             ]
 
     def iteritems(self):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         try:
             return obj.__dict__.iteritems()
         except Exception as e:
@@ -96,19 +102,19 @@ class DataObject(Mapping):
         return (k for k in self.keys())
 
     def __unicode__(self):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         return text(obj)
 
     def __str__(self):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         return str(obj)
 
     def __len__(self):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         return len(obj)
 
     def __call__(self, *args, **kwargs):
-        obj = _get(self, OBJ)
+        obj = _get(self, SLOT)
         return obj(*args, **kwargs)
 
 
@@ -132,8 +138,12 @@ def datawrap(v):
         return v
     elif type_ in generator_types:
         return (to_data(vv) for vv in v)
-    elif v == None:
-        return Null
+    try:
+        if v == None:
+            return Null
+    except Exception:
+        return DataObject(v)
+
     try:
         return v.__data__()
     except Exception:
